% Mesh rectangular prism according to the picture
% "Flattened_rectangular_prizm_mesh.eps"

% l - dimension in x direction
% t - dimension in y direction
% w - dimension in z direction
% Nl, Nt, Nw - number of intervals (number of nodes - 1) in x, y, z direction, respectively

% The origin is at the middle of face number 6.
% All faces and their numbers are shown on Figures 'Coordinate_system.esp'
% and 'Flattened_rectangular_prism_mesh.esp'

function [node_list facet_list]=mesh_rectangular_prism(l,t,w,Nl,Nt,Nw)

N1=(Nw+1)*(Nl+1);
N2=Nt*(Nl+1);
N3=Nw*(Nl+1);
N4=(Nt-1)*(Nl+1);
N5=(Nt-1)*(Nw-1);
N6=N5;

num_of_nodes=N1+N2+N3+N4+N5+N6;
num_of_facets=2*(2*Nl*Nt+2*Nl*Nw+2*Nt*Nw);

% Initialization
node_list=zeros(num_of_nodes,3);
facet_list=zeros(num_of_facets,3);

% Individual face lists of nodes
nodes1=zeros(N1,3);
nodes2=zeros(N2,3);
nodes3=zeros(N3,3);
nodes4=zeros(N4,3);
nodes5=zeros(N5,3);
nodes6=zeros(N6,3);

% Meshing individual faces
dx=l/Nl;
dy=t/Nt;
dz=w/Nw;
for i=1:N1
   nodes1(i,:)=[mod(i-1,Nl+1)*dx t/2 -w/2+floor((i-1)/(Nl+1))*dz]; 
end
for i=1:N2
   nodes2(i,:)=[mod(i-1,Nl+1)*dx t/2-dy-floor((i-1)/(Nl+1))*dy w/2]; 
end
for i=1:N3
   nodes3(i,:)=[mod(i-1,Nl+1)*dx -t/2 w/2-dz-floor((i-1)/(Nl+1))*dz]; 
end
for i=1:N4
   nodes4(i,:)=[mod(i-1,Nl+1)*dx -t/2+dy+floor((i-1)/(Nl+1))*dy -w/2]; 
end
for i=1:N5
   nodes5(i,:)=[l -t/2+dy+floor((i-1)/(Nw-1))*dy -w/2+dz+mod(i-1,Nw-1)*dz]; 
end
for i=1:N6
   nodes6(i,:)=[0 -t/2+dy+floor((i-1)/(Nw-1))*dy w/2-dz-mod(i-1,Nw-1)*dz]; 
end

% Combine node_list
node_list=[nodes1; nodes2; nodes3; nodes4; nodes5; nodes6];

% Create facets for individual faces
for i=1:Nl*Nw % For each rectangular cell, which contains 2 triangles
    
    % Face 1
    left_upper_node_num1=(Nl+1)*floor((i-1)/Nl)+mod(i-1,Nl)+1;
    facet_list(2*i-1,:)=[left_upper_node_num1 left_upper_node_num1+Nl+1 left_upper_node_num1+Nl+2];
    facet_list(2*i,:)=[left_upper_node_num1 left_upper_node_num1+1 left_upper_node_num1+Nl+2];
    
    % Face 3
    left_upper_node_num3=N1+N2-(Nl+1)+(Nl+1)*floor((i-1)/Nl)+mod(i-1,Nl)+1;
    facet_list(2*Nl*Nw+2*Nl*Nt+2*i-1,:)=[left_upper_node_num3 left_upper_node_num3+Nl+1 left_upper_node_num3+Nl+2];
    facet_list(2*Nl*Nw+2*Nl*Nt+2*i,:)=[left_upper_node_num3 left_upper_node_num3+1 left_upper_node_num3+Nl+2];
end
for i=1:Nl*Nt % For each rectangular cell, which contains 2 triangles
    
    % Face 2
    left_upper_node_num2=N1-(Nl+1)+(Nl+1)*floor((i-1)/Nl)+mod(i-1,Nl)+1;
    facet_list(2*Nl*Nw+2*i-1,:)=[left_upper_node_num2 left_upper_node_num2+Nl+1 left_upper_node_num2+Nl+2];
    facet_list(2*Nl*Nw+2*i,:)=[left_upper_node_num2 left_upper_node_num2+1 left_upper_node_num2+Nl+2];
    
    % Face 4
    left_upper_node_num4=N1+N2+N3-(Nl+1)+(Nl+1)*floor((i-1)/Nl)+mod(i-1,Nl)+1;
    facet_list(4*Nl*Nw+2*Nl*Nt+2*i-1,:)=mod([left_upper_node_num4 left_upper_node_num4+Nl+1 left_upper_node_num4+Nl+2]-1,N1+N2+N3+N4)+1;
    facet_list(4*Nl*Nw+2*Nl*Nt+2*i,:)=mod([left_upper_node_num4 left_upper_node_num4+1 left_upper_node_num4+Nl+2]-1,N1+N2+N3+N4)+1;
end

% Corner cells in faces 5 & 6
facet_list(4*Nl*Nw+4*Nl*Nt+1,:)=[N1+N2+N3 N1+N2+N3+(Nl+1) N1+N2+N3+N4+1]; % face 5 left upper corner
facet_list(4*Nl*Nw+4*Nl*Nt+2,:)=[N1+N2+N3 N1+N2+N3-(Nl+1) N1+N2+N3+N4+1];
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw-1,:)=[N1+N2+(Nl+1) N1+N2+N3+N4+(Nw-1) N1+N2-(Nl+1)]; % face 5 right upper corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw,:)=[N1+N2+(Nl+1) N1+N2 N1+N2-(Nl+1)];
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*(Nt-1)+1,:)=[N1+N2+N3+N4 Nl+1 2*(Nl+1)]; % face 5 left lower corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*(Nt-1)+2,:)=[N1+N2+N3+N4 N1+N2+N3+N4+(Nt-2)*(Nw-1)+1 2*(Nl+1)];
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*Nt-1,:)=[N1+N2+N3+N4+N5 Nw*(Nl+1) N1]; % face 5 right lower corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*Nt,:)=[N1+N2+N3+N4+N5 N1+(Nl+1) N1];

facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+1,:)=[N1+(Nt-1)*(Nl+1)+1 N1+(Nt-2)*(Nl+1)+1 N1+N2+N3+N4+N5+1]; % face 6 left upper corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2,:)=[N1+(Nt-1)*(Nl+1)+1 N1+N2+1 N1+N2+N3+N4+N5+1];
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw-1,:)=[N1+N2+(Nw-2)*(Nl+1)+1 N1+N2+N3+N4+N5+Nw-1 N1+N2+N3+1]; % face 6 right upper corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw,:)=[N1+N2+(Nw-2)*(Nl+1)+1 N1+N2+(Nw-1)*(Nl+1)+1 N1+N2+N3+1];
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw*(Nt-1)+1,:)=[N1+1 Nw*(Nl+1)+1 (Nw-1)*(Nl+1)+1]; % face 6 left lower corner
facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw*(Nt-1)+2,:)=[N1+1 N1+N2+N3+N4+N5+(Nt-2)*(Nw-1)+1 (Nw-1)*(Nl+1)+1];
facet_list(4*Nl*Nw+4*Nl*Nt+4*Nw*Nt-1,:)=[N1+N2+N3+N4+N5+N6 Nl+2 1]; % face 6 right lower corner
facet_list(4*Nl*Nw+4*Nl*Nt+4*Nw*Nt,:)=[N1+N2+N3+N4+N5+N6 N1+N2+N3+(Nt-2)*(Nl+1)+1 1];

for i=2:(Nw-1) % First and last row in faces 5 & 6

    % Face 5 - upper row
    left_upper_node_num5=N1+N2+N3-(i-1)*(Nl+1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*i-1,:)=[left_upper_node_num5 N1+N2+N3+N4+i-1 N1+N2+N3+N4+i];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*i,:)=[left_upper_node_num5 left_upper_node_num5-(Nl+1) N1+N2+N3+N4+i];
    
    % Face 5 - lower row
    left_upper_node_num5=N1+N2+N3+N4+(Nt-2)*(Nw-1)+i-1;
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*(Nt-1)+2*i-1,:)=[left_upper_node_num5 i*(Nl+1) (i+1)*(Nl+1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nw*(Nt-1)+2*i,:)=[left_upper_node_num5 left_upper_node_num5+1 (i+1)*(Nl+1)];
    
    % Face 6 - upper row
    left_upper_node_num6=N1+N2+1+(i-2)*(Nl+1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*i-1,:)=[left_upper_node_num6 N1+N2+N3+N4+N5+i-1 N1+N2+N3+N4+N5+i];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*i,:)=[left_upper_node_num6 left_upper_node_num6+(Nl+1) N1+N2+N3+N4+N5+i];

    % Face 6 - lower row
    left_upper_node_num6=N1+N2+N3+N4+N5+(Nt-2)*(Nw-1)+i-1;
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw*(Nt-1)+2*i-1,:)=[left_upper_node_num6 Nw*(Nl+1)+1-(i-1)*(Nl+1) Nw*(Nl+1)+1-i*(Nl+1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*Nw*(Nt-1)+2*i,:)=[left_upper_node_num6 left_upper_node_num6+1 Nw*(Nl+1)+1-i*(Nl+1)];

end

for i=2:(Nt-1) % First and last column in faces 5 & 6

    % Face 5 - left column
    left_upper_node_num5=N1+N2+N3+(i-1)*(Nl+1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*(i-1)*Nw+1,:)=[left_upper_node_num5 left_upper_node_num5+(Nl+1) N1+N2+N3+N4+1+(i-1)*(Nw-1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*(i-1)*Nw+2,:)=[left_upper_node_num5 N1+N2+N3+N4+1+(i-2)*(Nw-1) N1+N2+N3+N4+1+(i-1)*(Nw-1)];
    
    % Face 5 - right column
    left_upper_node_num5=N1+N2+N3+N4+(i-1)*(Nw-1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*i*Nw-1,:)=[left_upper_node_num5 left_upper_node_num5+(Nw-1) N1+N2-i*(Nl+1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*i*Nw,:)=[left_upper_node_num5 N1+N2-(i-1)*(Nl+1) N1+N2-i*(Nl+1)];
    
    % Face 6 - left column
    left_upper_node_num6=N1+(Nt-1)*(Nl+1)+1-(i-1)*(Nl+1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*(i-1)*Nw+1,:)=[left_upper_node_num6 left_upper_node_num6-(Nl+1) N1+N2+N3+N4+N5+1+(i-1)*(Nw-1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*(i-1)*Nw+2,:)=[left_upper_node_num6 N1+N2+N3+N4+N5+1+(i-2)*(Nw-1) N1+N2+N3+N4+N5+1+(i-1)*(Nw-1)];

    % Face 6 - right column
    left_upper_node_num6=N1+N2+N3+N4+N5+(i-1)*(Nw-1);
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*i*Nw-1,:)=[left_upper_node_num6 left_upper_node_num6+(Nw-1) N1+N2+(Nw-1)*(Nl+1)+1+i*(Nl+1)];
    facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*i*Nw,:)=[left_upper_node_num6 N1+N2+(Nw-1)*(Nl+1)+1+(i-1)*(Nl+1) N1+N2+(Nw-1)*(Nl+1)+1+i*(Nl+1)];

end

% Internal cells in faces 5 & 6
for j=2:(Nw-1) % for all internal columns
    for i=2:(Nt-1) % for all internal rows
        
        left_upper_node5=N1+N2+N3+N4+1+(i-2)*(Nw-1)+(j-2);
        facet_list(4*Nl*Nw+4*Nl*Nt+2*(i-1)*Nw+2*j-1,:)=[left_upper_node5 left_upper_node5+(Nw-1) left_upper_node5+Nw];
        facet_list(4*Nl*Nw+4*Nl*Nt+2*(i-1)*Nw+2*j,:)=[left_upper_node5 left_upper_node5+1 left_upper_node5+Nw];
        
        left_upper_node6=N1+N2+N3+N4+N5+1+(i-2)*(Nw-1)+(j-2);
        facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*(i-1)*Nw+2*j-1,:)=[left_upper_node6 left_upper_node6+(Nw-1) left_upper_node6+Nw];
        facet_list(4*Nl*Nw+4*Nl*Nt+2*Nt*Nw+2*(i-1)*Nw+2*j,:)=[left_upper_node6 left_upper_node6+1 left_upper_node6+Nw];
        
    end
end
