% Analytical integration of 1/|r-r0| over a triangle patch, following Graglia_93

% a,b,c - coordinates of 3 vertices, each is a vector 1X3
% area - area of the triangle
% r0 - test point coordinates, vector 1X3

function I=triangle_analytical_integration(a,b,c,area,r0)

n=cross((b-a),(c-a)); % Normal vector
% double_area=norm(n); % Double area of the triangle
double_area=2*area; % Double area of the triangle

l1=norm(c-b);
l2=norm(a-c);
l3=norm(b-a);

% Unit vectors of the local frame
ew=n/double_area;
eu=(b-a)/l3;
ev=cross(ew,(b-a)/l3);

% Local coordinates of node c
u3=(c-a)*eu';
v3=double_area/l3;

% Local coordinates of the test point
u0=(r0-a)*eu';
v0=(r0-a)*ev';
w0=(r0-a)*ew';

s1_minus=-((l3-u0)*(l3-u3)+v0*v3)/l1;
s1_plus=((u3-u0)*(u3-l3)+v3*(v3-v0))/l1;
s2_minus=-(u3*(u3-u0)+v3*(v3-v0))/l2;
s2_plus=(u0*u3+v0*v3)/l2;
s3_minus=-u0;
s3_plus=l3-u0;

t1_0=(v0*(u3-l3)+v3*(l3-u0))/l1;
t2_0=(u0*v3-v0*u3)/l2;
t3_0=v0;
t1_minus=sqrt((l3-u0)^2+v0^2);
t1_plus=sqrt((u3-u0)^2+(v3-v0)^2);
t2_minus=t1_plus;
t2_plus=sqrt(u0^2+v0^2);
t3_minus=t2_plus;
t3_plus=t1_minus;

if w0==0 % The test point lies in the plane of the triangle
    f21=log((t1_plus+s1_plus)/(t1_minus+s1_minus));
    f22=log((t2_plus+s2_plus)/(t2_minus+s2_minus));
    f23=log((t3_plus+s3_plus)/(t3_minus+s3_minus));
    
    I=t1_0*f21+t2_0*f22+t3_0*f23;
else
    w0_squared=w0^2;
    R1_0=sqrt(t1_0^2+w0_squared);
    R2_0=sqrt(t2_0^2+w0_squared);
    R3_0=sqrt(t3_0^2+w0_squared);

    R1_minus=sqrt(t1_minus^2+w0_squared);
    R2_minus=sqrt(t2_minus^2+w0_squared);
    R3_minus=sqrt(t3_minus^2+w0_squared);

    R1_plus=sqrt(t1_plus^2+w0_squared);
    R2_plus=sqrt(t2_plus^2+w0_squared);
    R3_plus=sqrt(t3_plus^2+w0_squared);

    f21=log((R1_plus+s1_plus)/(R1_minus+s1_minus));
    f22=log((R2_plus+s2_plus)/(R2_minus+s2_minus));
    f23=log((R3_plus+s3_plus)/(R3_minus+s3_minus));

    beta1=atan(t1_0*s1_plus/(R1_0^2+abs(w0)*R1_plus))-atan(t1_0*s1_minus/(R1_0^2+abs(w0)*R1_minus));
    beta2=atan(t2_0*s2_plus/(R2_0^2+abs(w0)*R2_plus))-atan(t2_0*s2_minus/(R2_0^2+abs(w0)*R2_minus));
    beta3=atan(t3_0*s3_plus/(R3_0^2+abs(w0)*R3_plus))-atan(t3_0*s3_minus/(R3_0^2+abs(w0)*R3_minus));
    beta=beta1+beta2+beta3;
    
    I=-abs(w0)*beta+t1_0*f21+t2_0*f22+t3_0*f23;
end